@@ -311,10 +311,18 @@ IFFChunk::ChunkList IFFChunk::innerFromDevice(QIODevice *d, bool *ok, IFFChunk *
             chunk = QSharedPointer<IFFChunk>(new ICCNChunk());
         } else if (cid == ICCP_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new ICCPChunk());
+        } else if (cid == IDAT_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new IDATChunk());
+        } else if (cid == IHDR_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new IHDRChunk());
+        } else if (cid == IPAR_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new IPARChunk());
         } else if (cid == NAME_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new NAMEChunk());
         } else if (cid == PCHG_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new PCHGChunk());
+        } else if (cid == PLTE_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new PLTEChunk());
         } else if (cid == RAST_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new RASTChunk());
         } else if (cid == RGBA_CHUNK) {
@@ -323,12 +331,16 @@ IFFChunk::ChunkList IFFChunk::innerFromDevice(QIODevice *d, bool *ok, IFFChunk *
             chunk = QSharedPointer<IFFChunk>(new SHAMChunk());
         } else if (cid == TBHD_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new TBHDChunk());
+        } else if (cid == VDAT_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new VDATChunk());
         } else if (cid == VERS_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new VERSChunk());
         } else if (cid == XBMI_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new XBMIChunk());
         } else if (cid == XMP0_CHUNK) {
             chunk = QSharedPointer<IFFChunk>(new XMP0Chunk());
+        } else if (cid == YUVS_CHUNK) {
+            chunk = QSharedPointer<IFFChunk>(new YUVSChunk());
         } else { // unknown chunk
             chunk = QSharedPointer<IFFChunk>(new IFFChunk());
             qCDebug(LOG_IFFPLUGIN) << "IFFChunk::innerFromDevice(): unknown chunk" << cid;
@@ -617,7 +629,7 @@ CAMGChunk::CAMGChunk() : IFFChunk()
 
 bool CAMGChunk::isValid() const
 {
-    if (bytes() != 4) {
+    if (dataBytes() != 4) {
         return false;
     }
     return chunkId() == CAMGChunk::defaultChunkId();
@@ -843,6 +855,23 @@ inline qint64 rgbNDecompress(QIODevice *input, char *output, qint64 olen)
     return j;
 }
 
+
+inline qint64 vdatDecompress(const IFFChunk *chunk, const BMHDChunk *header, qint32 y, char *output, qint64 olen)
+{
+    auto vdats = IFFChunk::searchT<VDATChunk>(chunk);
+    auto rowLen = header->rowLen();
+    if (olen < rowLen * vdats.size()) {
+        return -1;
+    }
+    for (qint32 i = 0, n = vdats.size(); i < n; ++i) {
+        auto&& uc = vdats.at(i)->uncompressedData(header);
+        if (y * rowLen > uc.size() - rowLen)
+            return -1;
+        std::memcpy(output + (i * rowLen), uc.data() + (y * rowLen), rowLen);
+    }
+    return vdats.size() * rowLen;
+}
+
 QByteArray BODYChunk::strideRead(QIODevice *d, qint32 y, const BMHDChunk *header, const CAMGChunk *camg, const CMAPChunk *cmap, const IPALChunk *ipal, const QByteArray& formType) const
 {
     if (!isValid() || header == nullptr || d == nullptr) {
@@ -868,6 +897,8 @@ QByteArray BODYChunk::strideRead(QIODevice *d, qint32 y, const BMHDChunk *header
             // WARNING: The online spec says it's the same as TIFF but that's
             // not accurate: the RLE -128 code is not a noop.
             rr = packbitsDecompress(d, buf.data(), buf.size(), true);
+        } else if (header->compression() == BMHDChunk::Compression::Vdat) {
+            rr = vdatDecompress(this, header, y, buf.data(), buf.size());
         } else if (header->compression() == BMHDChunk::Compression::RgbN8) {
             if (isRgb8)
                 rr = rgb8Decompress(d, buf.data(), buf.size());
@@ -914,11 +945,11 @@ CAMGChunk::ModeIds BODYChunk::safeModeId(const BMHDChunk *header, const CAMGChun
     auto cmapCount = cmap ? cmap->count() : 0;
     auto bitplanes = header->bitplanes();
     if (bitplanes >= BITPLANES_HALFBRIDE_MIN && bitplanes <= BITPLANES_HALFBRIDE_MAX) {
-        if (cmapCount == (1 << (header->bitplanes() - 1)))
+        if (cmapCount == (1 << (bitplanes - 1)))
             return CAMGChunk::ModeIds(CAMGChunk::ModeId::HalfBrite);
     }
     if (bitplanes >= BITPLANES_HAM_MIN && bitplanes <= BITPLANES_HAM_MAX) {
-        if (cmapCount == (1 << (header->bitplanes() - 2)))
+        if (cmapCount == (1 << (bitplanes - 2)))
             return CAMGChunk::ModeIds(CAMGChunk::ModeId::Ham);
     }
     return CAMGChunk::ModeIds();
@@ -974,6 +1005,15 @@ QByteArray BODYChunk::rgbN(const QByteArray &planes, qint32, const BMHDChunk *he
     return planes;
 }
 
+bool BODYChunk::innerReadStructure(QIODevice *d)
+{
+    auto ok = true;
+    if (d->peek(4) == VDAT_CHUNK) {
+        setChunks(IFFChunk::innerFromDevice(d, &ok, this));
+    }
+    return ok;
+}
+
 QByteArray BODYChunk::deinterleave(const QByteArray &planes, qint32 y, const BMHDChunk *header, const CAMGChunk *camg, const CMAPChunk *cmap, const IPALChunk *ipal) const
 {
     if (planes.size() != strideSize(header, ILBM_FORM_TYPE)) {
@@ -1406,16 +1446,13 @@ bool FORMChunk::innerReadStructure(QIODevice *d)
         setChunks(IFFChunk::innerFromDevice(d, &ok, this));
     } else if (_type == RGBN_FORM_TYPE) {
         setChunks(IFFChunk::innerFromDevice(d, &ok, this));
+    } else if (_type == IMAG_FORM_TYPE) {
+        setChunks(IFFChunk::innerFromDevice(d, &ok, this));
     }
     return ok;
 }
 
-QByteArray FORMChunk::formType() const
-{
-    return _type;
-}
-
-QImage::Format FORMChunk::format() const
+QImage::Format FORMChunk::iffFormat() const
 {
     auto headers = IFFChunk::searchT<BMHDChunk>(chunks());
     if (headers.isEmpty()) {
@@ -1464,13 +1501,67 @@ QImage::Format FORMChunk::format() const
     return QImage::Format_Invalid;
 }
 
+QImage::Format FORMChunk::cdiFormat() const
+{
+    auto headers = IFFChunk::searchT<IHDRChunk>(chunks());
+    if (headers.isEmpty()) {
+        return QImage::Format_Invalid;
+    }
+
+    if (auto &&h = headers.first()) {
+        if (h->model() == IHDRChunk::Rgb555 && h->depth() == 16) { // no test case
+            return QImage::Format_RGB555;
+        }
+
+        if (h->depth() == 4) {
+            if (h->model() == IHDRChunk::CLut4 || h->model() == IHDRChunk::CLut3) { // CLut3: no test case
+                return QImage::Format_Indexed8;
+            }
+        }
+
+        if (h->depth() == 8) {
+            if (h->model() == IHDRChunk::CLut8 || h->model() == IHDRChunk::CLut7 || h->model() == IHDRChunk::Rle7) {
+                return QImage::Format_Indexed8;
+            }
+            if (h->model() == IHDRChunk::Rgb888) { // no test case
+                return FORMAT_RGB_8BIT;
+            }
+            if (h->model() == IHDRChunk::DYuv) {
+                return FORMAT_RGB_8BIT;
+            }
+        }
+    }
+
+    return QImage::Format_Invalid;
+}
+
+QByteArray FORMChunk::formType() const
+{
+    return _type;
+}
+
+QImage::Format FORMChunk::format() const
+{
+    if (formType() == IMAG_FORM_TYPE) {
+        return cdiFormat();
+    }
+    return iffFormat();
+}
+
 QSize FORMChunk::size() const
 {
-    auto headers = IFFChunk::searchT<BMHDChunk>(chunks());
-    if (headers.isEmpty()) {
-        return {};
+    if (formType() == IMAG_FORM_TYPE) {
+        auto ihdrs = IFFChunk::searchT<IHDRChunk>(chunks());
+        if (!ihdrs.isEmpty()) {
+            return ihdrs.first()->size();
+        }
+    } else {
+        auto bmhds = IFFChunk::searchT<BMHDChunk>(chunks());
+        if (!bmhds.isEmpty()) {
+            return bmhds.first()->size();
+        }
     }
-    return headers.first()->size();
+    return {};
 }
 
 /* ******************
@@ -1583,6 +1674,8 @@ bool CATChunk::innerReadStructure(QIODevice *d)
         setChunks(IFFChunk::innerFromDevice(d, &ok, this));
     } else if (_type == RGBN_FORM_TYPE) {
         setChunks(IFFChunk::innerFromDevice(d, &ok, this));
+    } else if (_type == IMAG_FORM_TYPE) {
+        setChunks(IFFChunk::innerFromDevice(d, &ok, this));
     }
     return ok;
 }
@@ -1603,7 +1696,7 @@ TBHDChunk::TBHDChunk()
 
 bool TBHDChunk::isValid() const
 {
-    if (bytes() != 24 && bytes() != 32) {
+    if (dataBytes() != 24 && dataBytes() != 32) {
         return false;
     }
     return chunkId() == TBHDChunk::defaultChunkId();
@@ -1637,7 +1730,7 @@ QSize TBHDChunk::size() const
 
 qint32 TBHDChunk::left() const
 {
-    if (bytes() != 32) {
+    if (dataBytes() != 32) {
         return 0;
     }
     return i32(data().at(27), data().at(26), data().at(25), data().at(24));
@@ -1645,7 +1738,7 @@ qint32 TBHDChunk::left() const
 
 qint32 TBHDChunk::top() const
 {
-    if (bytes() != 32) {
+    if (dataBytes() != 32) {
         return 0;
     }
     return i32(data().at(31), data().at(30), data().at(29), data().at(28));
@@ -1877,7 +1970,7 @@ QImage RGBAChunk::compressedTile(QIODevice *d, const TBHDChunk *header) const
             }
         }
     } else if (bpc == 2) {
-        auto cs = header->channels();
+        auto cs = std::max(1, header->channels());
         if (cs < 4) { // alpha on 64-bit images must be 0xFF
             std::memset(img.bits(), 0xFF, img.sizeInBytes());
         }
@@ -2308,6 +2401,123 @@ bool NAMEChunk::innerReadStructure(QIODevice *d)
 }
 
 
+/* ******************
+ * *** VDAT Chunk ***
+ * ****************** */
+
+VDATChunk::~VDATChunk()
+{
+
+}
+
+VDATChunk::VDATChunk()
+{
+
+}
+
+bool VDATChunk::isValid() const
+{
+    return chunkId() == VDATChunk::defaultChunkId();
+}
+
+static QByteArray decompressVdat(const QByteArray &comp)
+{
+    QByteArray out;
+    auto ok = true;
+    auto cpos = 0;
+
+    auto readU16BE = [&](const QByteArray &src, int &pos, bool *ok) -> quint16 {
+        if (pos + 2 > src.size()) {
+            *ok = false;
+            return 0;
+        }
+        auto v = quint16((quint8(src[pos]) << 8) | quint8(src[pos + 1]));
+        pos += 2;
+        return v;
+    };
+
+    auto readI8 = [&](const QByteArray &src, int &pos, bool *ok) -> qint8 {
+        if (pos >= src.size()) {
+            *ok = false;
+            return 0;
+        }
+        return qint8(src[pos++]);
+    };
+
+    auto emitWord = [&](quint16 w) {
+        out.append(char(w & 0xFF));
+        out.append(char(w >> 8));
+    };
+
+    auto cmdCnt = readU16BE(comp, cpos, &ok);
+    if (!ok) {
+        return{};
+    }
+
+    // decode command stream
+    auto dpos = cmdCnt + (cmdCnt & 1);
+    for (auto n = cmdCnt; cpos < n && dpos < comp.size() && ok;) {
+        auto cmd = readI8(comp, cpos, &ok);
+        if (cmd == 0) {
+            auto count = readU16BE(comp, dpos, &ok);
+            for (auto i = 0; i < count; ++i)
+                emitWord(readU16BE(comp, dpos, &ok));
+        } else if (cmd == 1) {
+            auto count = readU16BE(comp, dpos, &ok);
+            auto value = readU16BE(comp, dpos, &ok);
+            for (auto i = 0; i < count; ++i)
+                emitWord(value);
+        } else if (cmd < 0) {
+            auto count = -qint32(cmd);
+            for (auto i = 0; i < count; ++i)
+                emitWord(readU16BE(comp, dpos, &ok));
+        } else {
+            auto count = quint16(cmd);
+            auto value = readU16BE(comp, dpos, &ok);
+            for (auto i = 0; i < count; ++i)
+                emitWord(value);
+        }
+        if (!ok) {
+            return{};
+        }
+    }
+    return out;
+}
+
+static QByteArray vdatToIlbmPlane(const QByteArray &vdatData, const BMHDChunk *header)
+{
+    QByteArray ba(vdatData.size(), char());
+    auto rowLen = header->rowLen();
+    for (auto x = 0, n = 0; x < rowLen; x += 2) {
+        for (auto y = 0, off = x, h = header->height(); y < h; y++, off += rowLen) {
+            if ((off + 1 >= ba.size()) || n + 1 >= vdatData.size()) {
+                return{};
+            }
+            ba[off + 1] = vdatData.at(n++);
+            ba[off] = vdatData.at(n++);
+        }
+    }
+    return ba;
+}
+
+const QByteArray &VDATChunk::uncompressedData(const BMHDChunk *header) const
+{
+    if (uncompressed.isEmpty()) {
+        auto tmp = decompressVdat(data());
+        if (tmp.size() == header->rowLen() * header->height()) {
+            uncompressed = vdatToIlbmPlane(tmp, header);
+        }
+    }
+    return uncompressed;
+}
+
+
+bool VDATChunk::innerReadStructure(QIODevice *d)
+{
+    return cacheData(d);
+}
+
+
 /* ******************
  * *** VERS Chunk ***
  * ****************** */
@@ -2371,6 +2581,472 @@ bool XMP0Chunk::innerReadStructure(QIODevice *d)
 }
 
 
+/* ******************
+ * *** IHDR Chunk ***
+ * ****************** */
+
+IHDRChunk::~IHDRChunk()
+{
+
+}
+
+IHDRChunk::IHDRChunk()
+{
+
+}
+
+bool IHDRChunk::isValid() const
+{
+    if (dataBytes() < 14) {
+        return false;
+    }
+    return chunkId() == IHDRChunk::defaultChunkId();
+}
+
+qint32 IHDRChunk::width() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(1), data().at(0)));
+}
+
+qint32 IHDRChunk::height() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(5), data().at(4)));
+}
+
+QSize IHDRChunk::size() const
+{
+    return QSize(width(), height());
+}
+
+qint32 IHDRChunk::lineSize() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(3), data().at(2)));
+}
+
+quint16 IHDRChunk::depth() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(9), data().at(8)));
+}
+
+IHDRChunk::Model IHDRChunk::model() const
+{
+    if (!isValid()) {
+        return IHDRChunk::Model::Invalid;
+    }
+    return IHDRChunk::Model(ui16(data().at(7), data().at(6)));
+}
+
+IHDRChunk::DYuvKind IHDRChunk::yuvKind() const
+{
+    if (!isValid()) {
+        return IHDRChunk::DYuvKind::One;
+    }
+    return IHDRChunk::DYuvKind(data().at(10));
+}
+
+IHDRChunk::Yuv IHDRChunk::yuvStart() const
+{
+    if (!isValid()) {
+        return{};
+    }
+    return Yuv(data().at(11), data().at(12), data().at(13));
+}
+
+bool IHDRChunk::innerReadStructure(QIODevice *d)
+{
+    return cacheData(d);
+}
+
+
+/* ******************
+ * *** IPAR Chunk ***
+ * ****************** */
+
+IPARChunk::~IPARChunk()
+{
+
+}
+
+IPARChunk::IPARChunk()
+{
+
+}
+
+bool IPARChunk::isValid() const
+{
+    if (dataBytes() < 22) {
+        return false;
+    }
+    return chunkId() == IPARChunk::defaultChunkId();
+}
+
+qint32 IPARChunk::xOffset() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(1), data().at(0)));
+}
+
+qint32 IPARChunk::yOffset() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(3), data().at(2)));
+}
+
+double IPARChunk::aspectRatio() const
+{
+    if (!isValid()) {
+        return 1;
+    }
+    if (auto xr = ui16(data().at(5), data().at(4))) {
+        auto yr = double(ui16(data().at(7), data().at(6)));
+        return yr / xr;
+    }
+    return 1;
+}
+
+qint32 IPARChunk::xPage() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(9), data().at(8)));
+}
+
+qint32 IPARChunk::yPage() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(11), data().at(10)));
+}
+
+qint32 IPARChunk::xGrub() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(13), data().at(12)));
+}
+
+qint32 IPARChunk::yGrub() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return qint32(ui16(data().at(15), data().at(14)));
+}
+
+IPARChunk::Rgb IPARChunk::mask() const
+{
+    if (!isValid()) {
+        return {};
+    }
+    return Rgb(data().at(16), data().at(17), data().at(18));
+}
+
+IPARChunk::Rgb IPARChunk::transparency() const
+{
+    if (!isValid()) {
+        return {};
+    }
+    return Rgb(data().at(19), data().at(20), data().at(21));
+}
+
+bool IPARChunk::innerReadStructure(QIODevice *d)
+{
+    return cacheData(d);
+}
+
+
+/* ******************
+ * *** PLTE Chunk ***
+ * ****************** */
+
+PLTEChunk::~PLTEChunk()
+{
+
+}
+
+PLTEChunk::PLTEChunk()
+{
+
+}
+
+bool PLTEChunk::isValid() const
+{
+    if (dataBytes() < 4) {
+        return false;
+    }
+    if (dataBytes() - 4 < total() * 3) {
+        return false;
+    }
+    return chunkId() == PLTEChunk::defaultChunkId();
+}
+
+qint32 PLTEChunk::count() const
+{
+    if (!isValid()) {
+        return 0;
+    }
+    return total() - count();
+}
+
+qint32 PLTEChunk::offset() const
+{
+    return qint32(ui16(data().at(1), data().at(0)));
+}
+
+qint32 PLTEChunk::total() const
+{
+    return qint32(ui16(data().at(3), data().at(2)));
+}
+
+QList<QRgb> PLTEChunk::innerPalette() const
+{
+    if (!isValid()) {
+        return{};
+    }
+    QList<QRgb> l;
+    auto &&d = data();
+    for (qint32 i = offset(), n = total(); i < n; ++i) {
+        auto i3 = 4 + i * 3;
+        l << qRgb(d.at(i3), d.at(i3 + 1), d.at(i3 + 2));
+    }
+    return l;
+}
+
+
+/* ******************
+ * *** YUVS Chunk ***
+ * ****************** */
+
+YUVSChunk::~YUVSChunk()
+{
+
+}
+
+YUVSChunk::YUVSChunk()
+{
+
+}
+
+bool YUVSChunk::isValid() const
+{
+    return chunkId() == YUVSChunk::defaultChunkId();
+}
+
+qint32 YUVSChunk::count() const
+{
+    return dataBytes() / 3;
+}
+
+IHDRChunk::Yuv YUVSChunk::yuvStart(qint32 y) const
+{
+    if (!isValid() || y >= count()) {
+        return{};
+    }
+    return IHDRChunk::Yuv(data().at(y * 3), data().at(y * 3 + 1), data().at(y * 3 + 2));
+}
+
+bool YUVSChunk::innerReadStructure(QIODevice *d)
+{
+    return cacheData(d);
+}
+
+
+/* ******************
+ * *** IDAT Chunk ***
+ * ****************** */
+
+IDATChunk::~IDATChunk()
+{
+
+}
+
+IDATChunk::IDATChunk()
+{
+
+}
+
+bool IDATChunk::isValid() const
+{
+    return chunkId() == IDATChunk::defaultChunkId();
+}
+
+/*!
+ * Converts a YUV pixel to RGB.
+ */
+inline IPARChunk::Rgb yuvToRgb(IHDRChunk::Yuv yuv) {
+    IPARChunk::Rgb rgb;
+
+    // Green Book Cap. V Par. 4.4.2.2
+    const auto b = yuv.y + (yuv.u - 128.) * 1.733;
+    const auto r = yuv.y + (yuv.v - 128.) * 1.371;
+    const auto g = (yuv.y - 0.299 * r - 0.114 * b) / 0.587;
+
+    rgb.r = quint8(std::clamp(r + 0.5, 0., 255.));
+    rgb.g = quint8(std::clamp(g + 0.5, 0., 255.));
+    rgb.b = quint8(std::clamp(b + 0.5, 0., 255.));
+
+    return rgb;
+}
+
+static QByteArray decompressRL7Row(QIODevice *device, int width) {
+    QByteArray row;
+    for (auto x = 0; x < width;) {
+        char b;
+        if (!device->getChar(&b)) {
+            return{};
+        }
+        if (b & 0x80) {
+            auto color = b & 0x7F;
+            if (!device->getChar(&b)) {
+                return{};
+            }
+            auto length = quint8(b);
+            if (length == 0) {
+                row.append(width - x, char(color));
+                x = width;
+            } else {
+                auto count = std::min(int(length), width - x);
+                row.append(count, char(color));
+                x += count;
+            }
+        } else {
+            row.append(b);
+            x++;
+        }
+    }
+    return row;
+}
+
+QByteArray IDATChunk::strideRead(QIODevice *d, qint32 y, const IHDRChunk *header, const IPARChunk *params, const YUVSChunk *yuvs) const
+{
+    Q_UNUSED(params)
+    if (!isValid() || header == nullptr || d == nullptr) {
+        return {};
+    }
+
+    auto read = strideSize(header);
+    for (auto nextPos = nextChunkPos(); !d->atEnd() && d->pos() < nextPos;) {
+        QByteArray rr;
+        if (header->model() == IHDRChunk::Rle7) {
+            rr = decompressRL7Row(d, header->width());
+        } else {
+            rr = d->read(read);
+        }
+
+        if (header->model() == IHDRChunk::CLut4) {
+            if (rr.size() < header->width() / 2) {
+                return {};
+            }
+            QByteArray tmp(header->width(), char());
+            for (auto x = 0, w = header->width(); x < w; ++x) {
+                auto i8 = quint8(rr.at(x / 2));
+                tmp[x] = x & 1 ? i8 & 0xF : (i8 >> 4) & 0xF;
+            }
+            rr = tmp;
+        }
+
+        if (header->model() == IHDRChunk::Rgb555) {
+            for (qint32 x = 0, w = rr.size() - 1; x < w; x += 2) {
+                std::swap(rr[x], rr[x + 1]);
+            }
+        }
+
+        if (header->model() == IHDRChunk::DYuv) {
+            if (rr.size() < header->width()) {
+                return {};
+            }
+
+            // delta table: Green Book Cap. V Par. 3.4.1.3
+            // NOTE 1: using the wrong delta table creates visible artifacts on the image.
+            // NOTE 2: using { 0, 1, 4, 9, 16, 27, 44, 79, -128, -79, -44, -27, -16, -9, -4, -1 }
+            //         table gives the same result (when assigned to an uint8).
+            static const qint32 deltaTable[16] = {
+                0, 1, 4, 9, 16, 27, 44, 79, 128, 177, 212, 229, 240, 247, 252, 255
+            };
+
+            auto yuv = header->yuvStart();
+            if (header->yuvKind() == IHDRChunk::Each && yuvs) {
+                yuv = yuvs->yuvStart(y);
+            }
+
+            QByteArray tmp(header->width() * 3, char());
+            for (auto x = 0, w = header->width() - 1; x < w; x += 2) {
+                // nibble order from Green Book Cap. V Par. 6.5.1.1
+                // NOTE: using the wrong nibble order creates visible artifacts on the image.
+                const auto du = deltaTable[(rr.at(x) >> 4) & 0x0F];
+                const auto d1 = deltaTable[rr.at(x) & 0x0F];
+                const auto dv = deltaTable[(rr.at(x + 1) >> 4) & 0x0F];
+                const auto d2 = deltaTable[rr.at(x + 1) & 0x0F];
+
+                // pixel 1
+                yuv.y = d1 + yuv.y;
+                yuv.u = du + yuv.u;
+                yuv.v = dv + yuv.v;
+                auto rgb = yuvToRgb(yuv);
+                tmp[x * 3] = rgb.r;
+                tmp[x * 3 + 1] = rgb.g;
+                tmp[x * 3 + 2] = rgb.b;
+
+                // pixel 2
+                yuv.y = d2 + yuv.y;
+                rgb = yuvToRgb(yuv);
+                tmp[(x + 1) * 3] = rgb.r;
+                tmp[(x + 1) * 3 + 1] = rgb.g;
+                tmp[(x + 1) * 3 + 2] = rgb.b;
+            }
+            rr = tmp;
+        }
+
+        return rr;
+    }
+
+    return {};
+}
+
+bool IDATChunk::resetStrideRead(QIODevice *d) const
+{
+    return seek(d);
+}
+
+quint32 IDATChunk::strideSize(const IHDRChunk *header) const
+{
+    if (header == nullptr) {
+        return 0;
+    }
+
+    auto rs = (header->width() * header->depth() + 7) / 8;
+
+    // No padding bytes are inserted in the data.
+    if (header->model() == IHDRChunk::Rgb888) {
+        return rs;
+    }
+
+    // The first pixel of each scan line must begin in a longword boundary.
+    if (auto mod = rs % 4)
+        rs += (4 - mod);
+    return rs;
+}
+
+
 /* ******************
  * *** BEAM Chunk ***
  * ****************** */
@@ -2727,41 +3403,38 @@ QList<QRgb> PCHGChunk::palette(qint32 y) const
 //   [ tree (treeSize bytes, even) | compressed bitstream (... bytes) ]
 //
 // On any error, logs with qCCritical(LOG_IFFPLUGIN) and returns {}.
-// Comments are in English as requested.
 // ----------------------------------------------------------------------------
 //
 // NOTE: Sebastiano Vigna, the author of the PCHG specification and the ASM
 //       decompression code for the Motorola 68K, gave us permission to use his
 //       code and recommended that we convert it with AI.
 
-// Read a big-endian 16-bit signed word from a byte buffer
-static inline qint16 read_be16(const char* base, int byteIndex, int size)
-{
-    if (byteIndex + 1 >= size)
-        return 0; // caller must bounds-check; we keep silent here
-    const quint8 b0 = static_cast<quint8>(base[byteIndex]);
-    const quint8 b1 = static_cast<quint8>(base[byteIndex + 1]);
-    return static_cast<qint16>((b0 << 8) | b1);
-}
-
-// Read a big-endian 32-bit unsigned long from a byte buffer
-static inline quint32 read_be32(const char* base, int byteIndex, int size)
-{
-    if (byteIndex + 3 >= size)
-        return 0; // caller must bounds-check
-    const quint8 b0 = static_cast<quint8>(base[byteIndex]);
-    const quint8 b1 = static_cast<quint8>(base[byteIndex + 1]);
-    const quint8 b2 = static_cast<quint8>(base[byteIndex + 2]);
-    const quint8 b3 = static_cast<quint8>(base[byteIndex + 3]);
-    return (static_cast<quint32>(b0) << 24) |
-           (static_cast<quint32>(b1) << 16) |
-           (static_cast<quint32>(b2) << 8)  |
-           static_cast<quint32>(b3);
-}
-
 // Core decompressor (tree + compressed stream in one QByteArray)
 static QByteArray pchgFastDecomp(const QByteArray& input, int treeSize, int originalSize)
 {
+    // Read a big-endian 16-bit signed word from a byte buffer
+    auto read_be16 = [&](const char* base, int byteIndex, int size) -> qint16 {
+        if (byteIndex + 1 >= size)
+            return 0; // caller must bounds-check; we keep silent here
+        const quint8 b0 = static_cast<quint8>(base[byteIndex]);
+        const quint8 b1 = static_cast<quint8>(base[byteIndex + 1]);
+        return static_cast<qint16>((b0 << 8) | b1);
+    };
+
+    // Read a big-endian 32-bit unsigned long from a byte buffer
+    auto read_be32 = [&](const char* base, int byteIndex, int size) -> quint32 {
+        if (byteIndex + 3 >= size)
+            return 0; // caller must bounds-check
+        const quint8 b0 = static_cast<quint8>(base[byteIndex]);
+        const quint8 b1 = static_cast<quint8>(base[byteIndex + 1]);
+        const quint8 b2 = static_cast<quint8>(base[byteIndex + 2]);
+        const quint8 b3 = static_cast<quint8>(base[byteIndex + 3]);
+        return (static_cast<quint32>(b0) << 24) |
+               (static_cast<quint32>(b1) << 16) |
+               (static_cast<quint32>(b2) << 8)  |
+                static_cast<quint32>(b3);
+    };
+
     // Basic validation
     if (treeSize <= 0 || (treeSize & 1)) {
         qCCritical(LOG_IFFPLUGIN) << "Invalid treeSize (must be positive and even)" << treeSize;
@@ -2888,7 +3561,6 @@ static QByteArray pchgFastDecomp(const QByteArray& input, int treeSize, int orig
 
     return out;
 }
-
 // !Huffman decompression
 
 bool PCHGChunk::initialize(const QList<QRgb> &cmapPalette, qint32 height)
